-- ##### TerraLAB - www.terralab.ufop.br
-- Computer Science Departament, Federal Univeristy of Ouro Preto
-- TerraME Prey-Predator dynamic model based on differental equations
-- Tiago Garcia de Senna Carneiro - 07/2012
-------------
-- You can improve model precision changing the timeStep (DT) or the integration method
-------------

--#PARAMETERS
birthPreyRate = 0.2
predationRate = 0.01 -- death prey rate
birthPredatorPerPreyRate = 0.01 -- birth predator rate
deathPredatorRate =  0.1 -- death prey rate


--#BEHAVIORAL MODEL
ag = Agent{ preys = 100, predators = 10 }

--#OBSERVERS
chartObserver = Observer{ subject = ag, type = "chart",attributes={"preys","predators"}, 
				title="Preys x Time", curveLabels={"preys", "predators"}, yLabel="#preys", xLabel="timer" }

	
--#Integration Method (please, vary it: "integrationHeun", "integrationRungeKutta" and 0.25, "integrationEuler" and 0.25)
INTEGRATION_METHOD =  integrationEuler

--#RULES
timeStep =  0.03125 -- See the effects of 0.25, 0.125, 0.0625, and 0.03125

for t = 0, 2000, timeStep do 

	ag.preys, ag.predators = d{
		{ 	
			function( t, q )  
				return q[1]*birthPreyRate - q[1]*q[2]*predationRate 
			end , 
			
			function( t, q ) 
				return q[2]*q[1]*birthPredatorPerPreyRate - q[2]*deathPredatorRate
			end
		},
		{ ag.preys, ag.predators }, 
		0, timeStep, 0.03125 
	}
	
	ag:notify(t)

end

print("Model outcome: ", ag.preys, ag.predators )


