-- ##### TerraLAB - www.terralab.ufop.br
-- Computer Science Departament, Federal Univeristy of Ouro Preto
-- TerraME Prey-Predator dynamic model based on differental equations
-- Tiago Garcia de Senna Carneiro - 07/2012
-------------
-- Lesson: Observe the phase space. You can use it to choose the proper integration method.
-------------

--#PARAMETERS
timeStep =  0.5 -- See the effects of 0.25, 0.125, 0.0625, and 0.03125
birthPreyRate = 0.2
predationRate = 0.01 -- death prey rate
birthPredatorPerPreyRate = 0.01 -- birth predator rate
deathPredatorRate =  0.1 -- death prey rate

--#Integration Method (please, vary it: ("integrationHeun"and 0.25), "integrationRungeKutta" and 0.03125, "integrationEuler" and 0.03125)
INTEGRATION_METHOD =  integrationEuler


--#BEHAVIORAL MODEL
ag = Agent{ preys = 100, predators = 10 }

--#OBSERVERS
 Observer{ subject = ag, type = "chart", attributes={"predators"}, xAxis = "preys", 
				title="phase space", curveLabels={"preys x predators"}, xLabel="#preys", yLabel="#predators" }

Observer{ subject = ag, type = "chart",attributes={"preys","predators"}, 
				title="Preys x Time", curveLabels={"preys", "predators"}, yLabel="#preys", xLabel="time" }
					

--#RULES
for t = 0, 10000, timeStep do 

	ag.preys, ag.predators = d{
		{ 	
			function( t, q )  
				return q[1]*birthPreyRate - q[1]*q[2]*predationRate 
			end , 
			
			function( t, q ) 
				return q[2]*q[1]*birthPredatorPerPreyRate - q[2]*deathPredatorRate
			end
		},
		{ ag.preys, ag.predators }, 
		0, timeStep, 0.03125 
	}
	
	print(t)
	ag:notify(t)

end

print("Model outcome: ", ag.preys, ag.predators )


